@@ -28,22 +28,22 @@ asmlinkage void poly1305_asm_2block_sse2(u32 *h, const u8 *src, const u32 *r, un
 #ifdef CONFIG_AS_AVX2
 asmlinkage void poly1305_asm_4block_avx2(u32 *h, const u8 *src, const u32 *r, unsigned int blocks, const u32 *u);
 #endif
-static bool chacha20poly1305_use_avx2 __read_mostly = false;
-static bool chacha20poly1305_use_ssse3 __read_mostly = false;
-static bool chacha20poly1305_use_sse2 __read_mostly = false;
+static bool chacha20poly1305_use_avx2 __read_mostly;
+static bool chacha20poly1305_use_ssse3 __read_mostly;
+static bool chacha20poly1305_use_sse2 __read_mostly;
 void chacha20poly1305_fpu_init(void)
 {
 	chacha20poly1305_use_sse2 = boot_cpu_has(X86_FEATURE_XMM2);
 	chacha20poly1305_use_ssse3 = boot_cpu_has(X86_FEATURE_SSSE3);
-	chacha20poly1305_use_avx2 = boot_cpu_has(X86_FEATURE_AVX) && boot_cpu_has(X86_FEATURE_AVX2);
+	chacha20poly1305_use_avx2 = boot_cpu_has(X86_FEATURE_AVX) && boot_cpu_has(X86_FEATURE_AVX2) && cpu_has_xfeatures(XFEATURE_MASK_SSE | XFEATURE_MASK_YMM, NULL);
 }
 #elif IS_ENABLED(CONFIG_KERNEL_MODE_NEON)
 #include <asm/hwcap.h>
 #include <asm/neon.h>
 asmlinkage void chacha20_asm_block_xor_neon(u32 *state, u8 *dst, const u8 *src);
 asmlinkage void chacha20_asm_4block_xor_neon(u32 *state, u8 *dst, const u8 *src);
-static bool chacha20poly1305_use_neon __read_mostly = false;
-void chacha20poly1305_fpu_init(void)
+static bool chacha20poly1305_use_neon __read_mostly;
+void __init chacha20poly1305_fpu_init(void)
 {
 #if defined(CONFIG_ARM64)
 	chacha20poly1305_use_neon = elf_hwcap & HWCAP_ASIMD;
@@ -52,7 +52,7 @@ void chacha20poly1305_fpu_init(void)
 #endif
 }
 #else
-void chacha20poly1305_fpu_init(void) { }
+void __init chacha20poly1305_fpu_init(void) { }
 #endif
 
 #define CHACHA20_IV_SIZE	16
@@ -458,8 +458,9 @@ static void poly1305_simd_mult(u32 *a, const u32 *b)
 
 	memset(m, 0, sizeof(m));
 	/* The poly1305 block function adds a hi-bit to the accumulator which
-	 * we don't need for key multiplication; compensate for it. */
-	a[4] -= 1 << 24;
+	 * we don't need for key multiplication; compensate for it.
+	 */
+	a[4] -= 1U << 24;
 	poly1305_asm_block_sse2(a, m, b, 1);
 }
 
@@ -523,7 +524,7 @@ static void poly1305_update(struct poly1305_ctx *ctx, const u8 *src, unsigned in
 				poly1305_simd_blocks(ctx, ctx->buf, POLY1305_BLOCK_SIZE);
 			else
 #endif
-				poly1305_generic_blocks(ctx, ctx->buf, POLY1305_BLOCK_SIZE, 1 << 24);
+				poly1305_generic_blocks(ctx, ctx->buf, POLY1305_BLOCK_SIZE, 1U << 24);
 			ctx->buflen = 0;
 		}
 	}
@@ -534,7 +535,7 @@ static void poly1305_update(struct poly1305_ctx *ctx, const u8 *src, unsigned in
 			bytes = poly1305_simd_blocks(ctx, src, srclen);
 		else
 #endif
-			bytes = poly1305_generic_blocks(ctx, src, srclen, 1 << 24);
+			bytes = poly1305_generic_blocks(ctx, src, srclen, 1U << 24);
 		src += srclen - bytes;
 		srclen = bytes;
 	}
@@ -574,10 +575,10 @@ static void poly1305_finish(struct poly1305_ctx *ctx, u8 *dst)
 
 	/* compute h + -p */
 	g0 = h0 + 5;
-	g1 = h1 + (g0 >> 26);             g0 &= 0x3ffffff;
-	g2 = h2 + (g1 >> 26);             g1 &= 0x3ffffff;
-	g3 = h3 + (g2 >> 26);             g2 &= 0x3ffffff;
-	g4 = h4 + (g3 >> 26) - (1 << 26); g3 &= 0x3ffffff;
+	g1 = h1 + (g0 >> 26);              g0 &= 0x3ffffff;
+	g2 = h2 + (g1 >> 26);              g1 &= 0x3ffffff;
+	g3 = h3 + (g2 >> 26);              g2 &= 0x3ffffff;
+	g4 = h4 + (g3 >> 26) - (1U << 26); g3 &= 0x3ffffff;
 
 	/* select h if h < p, or h + -p if h >= p */
 	mask = (g4 >> ((sizeof(u32) * 8) - 1)) - 1;
@@ -663,6 +664,7 @@ void chacha20poly1305_encrypt(u8 *dst, const u8 *src, const size_t src_len,
 			      const u64 nonce, const u8 key[CHACHA20POLY1305_KEYLEN])
 {
 	bool have_simd;
+
 	have_simd = chacha20poly1305_init_simd();
 	__chacha20poly1305_encrypt(dst, src, src_len, ad, ad_len, nonce, key, have_simd);
 	chacha20poly1305_deinit_simd(have_simd);
@@ -696,6 +698,7 @@ bool chacha20poly1305_encrypt_sg(struct scatterlist *dst, struct scatterlist *sr
 		ret = blkcipher_walk_virt_block(&chacha20_desc, &walk, CHACHA20_BLOCK_SIZE);
 		while (walk.nbytes >= CHACHA20_BLOCK_SIZE) {
 			size_t chunk_len = rounddown(walk.nbytes, CHACHA20_BLOCK_SIZE);
+
 			chacha20_crypt(&chacha20_state, walk.dst.virt.addr, walk.src.virt.addr, chunk_len, have_simd);
 			poly1305_update(&poly1305_state, walk.dst.virt.addr, chunk_len, have_simd);
 			ret = blkcipher_walk_done(&chacha20_desc, &walk, walk.nbytes % CHACHA20_BLOCK_SIZE);
@@ -780,6 +783,7 @@ bool chacha20poly1305_decrypt(u8 *dst, const u8 *src, const size_t src_len,
 			      const u64 nonce, const u8 key[CHACHA20POLY1305_KEYLEN])
 {
 	bool have_simd, ret;
+
 	have_simd = chacha20poly1305_init_simd();
 	ret = __chacha20poly1305_decrypt(dst, src, src_len, ad, ad_len, nonce, key, have_simd);
 	chacha20poly1305_deinit_simd(have_simd);
@@ -821,6 +825,7 @@ bool chacha20poly1305_decrypt_sg(struct scatterlist *dst, struct scatterlist *sr
 		ret = blkcipher_walk_virt_block(&chacha20_desc, &walk, CHACHA20_BLOCK_SIZE);
 		while (walk.nbytes >= CHACHA20_BLOCK_SIZE) {
 			size_t chunk_len = rounddown(walk.nbytes, CHACHA20_BLOCK_SIZE);
+
 			poly1305_update(&poly1305_state, walk.src.virt.addr, chunk_len, have_simd);
 			chacha20_crypt(&chacha20_state, walk.dst.virt.addr, walk.src.virt.addr, chunk_len, have_simd);
 			ret = blkcipher_walk_done(&chacha20_desc, &walk, walk.nbytes % CHACHA20_BLOCK_SIZE);
@@ -863,6 +868,7 @@ void xchacha20poly1305_encrypt(u8 *dst, const u8 *src, const size_t src_len,
 {
 	bool have_simd = chacha20poly1305_init_simd();
 	u8 derived_key[CHACHA20POLY1305_KEYLEN] __aligned(16);
+
 	hchacha20(derived_key, nonce, key, have_simd);
 	__chacha20poly1305_encrypt(dst, src, src_len, ad, ad_len, le64_to_cpuvp(nonce + 16), derived_key, have_simd);
 	memzero_explicit(derived_key, CHACHA20POLY1305_KEYLEN);
@@ -876,6 +882,7 @@ bool xchacha20poly1305_decrypt(u8 *dst, const u8 *src, const size_t src_len,
 {
 	bool ret, have_simd = chacha20poly1305_init_simd();
 	u8 derived_key[CHACHA20POLY1305_KEYLEN] __aligned(16);
+
 	hchacha20(derived_key, nonce, key, have_simd);
 	ret = __chacha20poly1305_decrypt(dst, src, src_len, ad, ad_len, le64_to_cpuvp(nonce + 16), derived_key, have_simd);
 	memzero_explicit(derived_key, CHACHA20POLY1305_KEYLEN);
